/*
 *  Copyright (C) 2000 Marco Pesenti Gritti
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "galeon.h"

/** The temp bookmarks window */
bookmarks_editor_controls *temp_bookmarks_window = NULL;

/** The bookmarks editing controls */
bookmarks_editor_controls *bookmarks_editor = NULL;

typedef enum {
	STANDARD_BOOKMARKS_EDITOR,
	COMPACT_BOOKMARKS_EDITOR,
	NEW_BOOKMARKS_EDITOR
} bookmarks_editor_controls_type;		

/* Function prototypes */
bookmarks_editor_controls *
bookmarks_editor_controls_init (BookMarkItem *root,
				bookmarks_editor_controls_type type);
void bookmarks_editor_remove_tree_item_null (BookMarkItem *b);
void bookmarks_editor_goto_bookmark (BookMarkItem *b);


bookmarks_editor_controls *
bookmarks_editor_controls_init (BookMarkItem *root,
				bookmarks_editor_controls_type type)
{
	GladeXML *gxml = NULL;
	GtkWidget *bookmarks_editor_edit_controls;
	bookmarks_editor_controls *editor;
		
	editor = g_new0 (bookmarks_editor_controls, 1);	

	bookmarks_load_icons ();
	
	editor->root_bookmark = root;

	switch (type) {
	case STANDARD_BOOKMARKS_EDITOR:
		gxml = glade_widget_new ("bookmarks_editor", editor);
		editor->dialog = glade_xml_get_widget 
			(gxml, "bookmarks_editor");
		editor->ctree = glade_xml_get_widget
			(gxml, "bookmarks_editor_ctree");
		editor->vpane = glade_xml_get_widget
			(gxml, "vpaned1");
		editor->edit_frame = glade_xml_get_widget
			(gxml, "bookmarks_editor_edit_frame");
		editor->name_entry = glade_xml_get_widget
			(gxml, "bookmarks_editor_name_entry");
		editor->url_entry = glade_xml_get_widget
			(gxml, "bookmarks_editor_url_entry");
		editor->nick_entry = glade_xml_get_widget
			(gxml, "bookmarks_editor_nick_entry");
		editor->pixmap_entry = glade_xml_get_widget
			(gxml, "bookmarks_editor_pixmap_entry");
		editor->pixmap_file_entry = glade_xml_get_widget
			(gxml, "bookmarks_editor_pixmap_file_entry");
		editor->create_toolbar_toggle = glade_xml_get_widget
			(gxml, "bookmarks_editor_create_toolbar_toggle");
		editor->create_context_menu_toggle = glade_xml_get_widget
			(gxml, "bookmarks_editor_create_context_menu_toggle");
		editor->notes_text = glade_xml_get_widget
			(gxml, "bookmarks_editor_notes_text");
		g_assert (editor->dialog != NULL);
		g_assert (editor->edit_frame != NULL);
		g_assert (editor->name_entry != NULL);
		g_assert (editor->url_entry != NULL);
		g_assert (editor->nick_entry != NULL);
		g_assert (editor->pixmap_entry != NULL);
		g_assert (editor->pixmap_file_entry != NULL);
		g_assert (editor->create_toolbar_toggle != NULL);
		g_assert (editor->create_context_menu_toggle != NULL);
		g_assert (editor->notes_text != NULL);
		break;
	case COMPACT_BOOKMARKS_EDITOR:
		gxml = glade_widget_new ("compact_bookmarks_editor", editor);
		editor->dialog = glade_xml_get_widget 
			(gxml, "compact_bookmarks_editor");
		editor->ctree = glade_xml_get_widget
			(gxml, "compact_bookmarks_editor_ctree");
		editor->edit_frame = glade_xml_get_widget
			(gxml, "compact_bookmarks_editor_edit_frame");
		editor->name_entry = glade_xml_get_widget
			(gxml, "compact_bookmarks_editor_name_entry");
		editor->url_entry = glade_xml_get_widget
			(gxml, "compact_bookmarks_editor_url_entry");
		editor->nick_entry = NULL;
		editor->pixmap_entry = NULL;
		editor->pixmap_file_entry = NULL;
		editor->create_toolbar_toggle = NULL;
		editor->create_context_menu_toggle = NULL;
		editor->notes_text = glade_xml_get_widget
			(gxml, "compact_bookmarks_editor_notes_text");
		g_assert (editor->dialog != NULL);
		g_assert (editor->edit_frame != NULL);
		g_assert (editor->name_entry != NULL);
		g_assert (editor->url_entry != NULL);
		g_assert (editor->nick_entry == NULL);
		g_assert (editor->pixmap_entry == NULL);
		g_assert (editor->pixmap_file_entry == NULL);
		g_assert (editor->create_toolbar_toggle == NULL);
		g_assert (editor->create_context_menu_toggle == NULL);
		g_assert (editor->notes_text != NULL);
		bookmarks_editor_edit_controls = glade_xml_get_widget
			(gxml, "compact_bookmarks_editor_control_buttons");
		gtk_toolbar_set_style (GTK_TOOLBAR (bookmarks_editor_edit_controls),
				       GTK_TOOLBAR_ICONS);
		break;
	case NEW_BOOKMARKS_EDITOR:
		gxml = glade_widget_new ("new_bookmarks_editor", editor);
		editor->dialog = glade_xml_get_widget 
			(gxml, "new_bookmarks_editor");
		editor->toolbar = glade_xml_get_widget 
			(gxml, "new_bookmarks_editor_toolbar");
		editor->ctree = glade_xml_get_widget
			(gxml, "new_bookmarks_editor_ctree");
		editor->vpane = NULL;
		editor->edit_frame = NULL;
		editor->name_entry = glade_xml_get_widget
			(gxml, "new_bookmarks_editor_name_entry");
		editor->url_entry = glade_xml_get_widget
			(gxml, "new_bookmarks_editor_url_entry");
		editor->nick_entry = glade_xml_get_widget
			(gxml, "new_bookmarks_editor_nick_entry");
		editor->pixmap_entry = glade_xml_get_widget
			(gxml, "new_bookmarks_editor_pixmap_entry");
		editor->pixmap_file_entry = glade_xml_get_widget
			(gxml, "new_bookmarks_editor_pixmap_file_entry");
		editor->create_toolbar_toggle = glade_xml_get_widget
			(gxml, "new_bookmarks_editor_create_toolbar_toggle");
		editor->create_context_menu_toggle = glade_xml_get_widget
			(gxml, "new_bookmarks_editor_create_context_menu_toggle");
		editor->notes_text = glade_xml_get_widget
			(gxml, "new_bookmarks_editor_notes_text");
		g_assert (editor->dialog != NULL);
		g_assert (editor->toolbar != NULL);
		g_assert (editor->name_entry != NULL);
		g_assert (editor->url_entry != NULL);
		g_assert (editor->nick_entry != NULL);
		g_assert (editor->pixmap_entry != NULL);
		g_assert (editor->pixmap_file_entry != NULL);
		g_assert (editor->create_toolbar_toggle != NULL);
		g_assert (editor->create_context_menu_toggle != NULL);
		g_assert (editor->notes_text != NULL);
		break;
	default:
		g_error ("Incorrect bookmarks editor dialog type");
	}
	g_assert (editor->ctree != NULL);
	
	editor->selection = NULL;
	editor->last_pressed = NULL;
	editor->lock_edit_controls = FALSE;
	editor->dirty = FALSE;
	editor->embed = NULL;
	editor->save_function = NULL;

	/* unref the GladeXML object, we don't use it anymore */
	gtk_object_unref (GTK_OBJECT (gxml));

	gtk_drag_source_set (editor->ctree, GDK_BUTTON1_MASK, 
			     bookmarks_dnd_targets, 
			     bookmarks_dnd_targets_num_items, 
			     GDK_ACTION_COPY | GDK_ACTION_MOVE |
			     GDK_ACTION_LINK);
	gtk_drag_dest_set (editor->ctree,
			   GTK_DEST_DEFAULT_DROP,
			   bookmarks_dnd_targets,
			   bookmarks_dnd_targets_num_items,
			   GDK_ACTION_COPY | GDK_ACTION_MOVE |
			   GDK_ACTION_LINK);

	if (editor->pixmap_entry != NULL)
	{
		gnome_pixmap_entry_set_pixmap_subdir 
			(GNOME_PIXMAP_ENTRY (editor->pixmap_entry), SHARE_DIR);
	}

	bookmarks_editor_place_tree_item (editor, root);
	return editor;
}

void 
bookmarks_editor_init (void)
{
	bookmarks_editor = bookmarks_editor_controls_init
		(bookmarks_root, NEW_BOOKMARKS_EDITOR);
	bookmarks_editor->save_function = bookmarks_save;
}

/**
 * Inits the temp bookmarks
 */
void
temp_bookmarks_init (void)
{
	char *file = g_strconcat (g_get_home_dir(), "/.galeon/temp_bookmarks.xml", NULL);
	temp_bookmarks_root = bookmarks_load_from (file);
	if (!temp_bookmarks_root) 
		temp_bookmarks_root = bookmarks_new_bookmark 
			(CATEGORY, _("Temporary bookmarks"), 
			 NULL, NULL, NULL, NULL);
	temp_bookmarks_window = bookmarks_editor_controls_init 
		(temp_bookmarks_root, COMPACT_BOOKMARKS_EDITOR);
	temp_bookmarks_window->save_function = temp_bookmarks_save;
}
				
/**
 * bookmarks_editor_place_tree_item: place a bookmark in the editor ctree
 * @controls: the set of controls
 * @b: the bookmark
 * 
 * Create the tree items for a bookmark and its children. If it has already
 * been placed, removes the old item.
 * 
 **/
void
bookmarks_editor_place_tree_item (bookmarks_editor_controls *controls, BookMarkItem *b)
{
	gchar *text[3];
	gboolean isleaf;
	GdkPixbuf *bm_icon;
	GdkPixmap *pixmap;
	GdkPixmap *pixmap_open;
	GdkPixmap *mask_open;
	GdkBitmap *mask;
	GtkCTreeNode *parent_node;
	GtkCTreeNode *sibling_node;

	text[2] = NULL;
	switch (b->type) {
	case SEPARATOR:
		text[0] = text[1] = "------";
		isleaf = TRUE;
		pixmap = mask = pixmap_open = mask_open = NULL;
		break;
	case SITE:
		text[0] = b->name;
		text[1] = b->url;
		isleaf = TRUE;

		bm_icon = bookmarks_get_siteicon (b->url,  FALSE);
		gdk_pixbuf_render_pixmap_and_mask (bm_icon, &pixmap, &mask, 100);
		gdk_pixbuf_unref (bm_icon);

		pixmap_open = pixmap;
		mask_open = mask;
		break;
	case CATEGORY:
	case AUTOBOOKMARKS:
		text[0] = b->name;
		text[1] = "";
		isleaf = FALSE;
		gdk_pixbuf_render_pixmap_and_mask (folder_pixmap, 
						   &pixmap, &mask, 100);
		gdk_pixbuf_render_pixmap_and_mask (folder_open_pixmap, 
						&pixmap_open, &mask_open, 100);
		break;
	default:
		text[0] = text[1] = "";
		isleaf = TRUE;
		pixmap = mask = pixmap_open = mask_open = NULL;
		break;
	}

	if ((b->parent == NULL) || (b->parent == controls->root_bookmark))
		parent_node = NULL;
	else
		parent_node = b->parent->tree_item;

	sibling_node = NULL;
	if (b->parent != NULL) {
		GList *list_node = g_list_find (b->parent->list, b);
		if (list_node) {
			GList *sibling_list_node = g_list_next (list_node);
			if (sibling_list_node) {
				sibling_node = ((BookMarkItem *) 
					(sibling_list_node->data))->tree_item;
			}
		}
	}
	gtk_clist_freeze (GTK_CLIST (controls->ctree));
	if (b->tree_item)
		bookmarks_editor_remove_tree_item (controls, b);
	if (b != controls->root_bookmark) {
		b->tree_item = gtk_ctree_insert_node
			(GTK_CTREE (controls->ctree), parent_node, sibling_node, text, 
			 2, pixmap, mask, pixmap_open, mask_open, isleaf, b->expanded);
		gtk_ctree_node_set_row_data (GTK_CTREE (controls->ctree), 
					     b->tree_item, b);
	}

	if (!isleaf) {
		GList *l;
		for (l = b->list; l != NULL; l = g_list_next (l))
			bookmarks_editor_place_tree_item (controls, l->data);
	}
	gtk_clist_thaw (GTK_CLIST (controls->ctree));
}

/**
 * bookmarks_editor_remove_tree_item: remove a bookmark from the editor ctree
 * @controls: the set of controls
 * @b: the bookmark
 * 
 * Removes bookmarks and it's childs from the bookmark editor ctree and sets the 
 * tree_item fields to NULL. This does not delete the bookmarks
 * 
 **/
void
bookmarks_editor_remove_tree_item (bookmarks_editor_controls *controls, BookMarkItem *b)
{
	/* this removes the treeitem and its children */
	gtk_ctree_remove_node (GTK_CTREE (controls->ctree), b->tree_item);
	/* now, let's NULL the fields */
	bookmarks_editor_remove_tree_item_null (b);
}

/**
 * bookmarks_editor_remove_tree_item_null: Sets to NULL the tree_item field of
 * a bookmarks and its children
 * @b: the bookmark
 * 
 * This is an auxiliar function for bookmarks_editor_remove_tree_item.
 **/
void
bookmarks_editor_remove_tree_item_null (BookMarkItem *b)
{
	GList *l;
	b->tree_item = NULL;
	if (b->type == CATEGORY) 
		for (l = b->list; l != NULL; l = g_list_next (l)) 
			bookmarks_editor_remove_tree_item_null (l->data);
}

/**
 * bookmarks_editor_update_tree_item: update a bookmark in the editor ctree
 * @controls: the set of controls
 * @b: the bookmark
 * 
 * Updates the bookmark (or adds it if not yet added)
 * 
 **/
void
bookmarks_editor_update_tree_item (bookmarks_editor_controls *controls, BookMarkItem *b)
{
	gchar *text[3];
	gboolean isleaf;
	GdkPixbuf *bm_icon;
	GdkPixmap *pixmap;
	GdkBitmap *mask;
	GtkCTreeNode *parent_node;
		
	if (!b->tree_item) {
		bookmarks_editor_place_tree_item (controls, b);
		return;
	}

	text[2] = NULL;
	switch (b->type) {
	case SEPARATOR:
		text[0] = text[1] = "------";
		isleaf = TRUE;
		pixmap = mask = NULL;
		break;
	case SITE:
		text[0] = b->name;
		text[1] = b->url;
		isleaf = TRUE;
		bm_icon = bookmarks_get_siteicon (b->url,  FALSE);
		gdk_pixbuf_render_pixmap_and_mask (bm_icon, &pixmap, &mask, 100);
		gdk_pixbuf_unref (bm_icon);
		pixmap = pixmap;
		mask = mask;
		break;
	case CATEGORY:
	case AUTOBOOKMARKS:
		text[0] = b->name;
		text[1] = "";
		isleaf = FALSE;
		gdk_pixbuf_render_pixmap_and_mask (folder_pixmap, 
						   &pixmap, &mask, 100);
		break;
	default:
		text[0] = text[1] = "";
		isleaf = TRUE;
		pixmap = mask = NULL;
		break;
	}

	if ((b->parent == NULL) || (b->parent == controls->root_bookmark))
		parent_node = NULL;
	else
		parent_node = b->parent->tree_item;

	if (b != controls->root_bookmark) {
		gtk_ctree_node_set_pixtext
			(GTK_CTREE (controls->ctree), b->tree_item, 0, text[0],
			2, pixmap, mask);
		gtk_ctree_node_set_text
			(GTK_CTREE (controls->ctree), b->tree_item, 1, text[1]);
	}
}

/**
 * bookmarks_editor_goto_bookmark : position the bookmark editor 
 * 	at a given bookmark item & focus the name entry
 * @b : bookmark item to focus
 */
void bookmarks_editor_goto_bookmark (BookMarkItem *b)
{
	GtkCTreeNode *pnode;
	GtkCTree *ctree = GTK_CTREE (bookmarks_editor->ctree);
	GtkCList *clist = GTK_CLIST (bookmarks_editor->ctree);
	gint row;
	
	g_return_if_fail (bookmarks_editor != NULL);
	
	pnode = b->tree_item;
	while (pnode)
	{
		gtk_ctree_expand (ctree, pnode);
		pnode = GTK_CTREE_ROW (pnode)->parent;
	}

	gtk_ctree_node_moveto (ctree, b->tree_item, 0, 0, 0);
	gtk_clist_unselect_all (clist);
	gtk_ctree_select (ctree, b->tree_item);
	row = g_list_position (clist->row_list, (GList *) b->tree_item);
	clist->focus_row = row;
	gtk_widget_grab_focus (bookmarks_editor->name_entry);
}
