/*
 *  Copyright (C) 2000 Marco Pesenti Gritti
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "galeon.h"

static GtkWidget *bookmarks_folder_context_menu_new (gboolean create_toolbar);
void bookmarks_import_mozilla_activate_cb (GtkMenuItem *menuitem,
					   GaleonWindow *window);
void bookmarks_import_other_activate_cb (GtkMenuItem *menuitem,
					 GaleonWindow *window);
void bookmarks_export_netscape_activate_cb (GtkMenuItem *menuitem, 
					    GaleonWindow *window);
void bookmarks_export_mozilla_activate_cb (GtkMenuItem *menuitem,
					   GaleonWindow *window);
void bookmarks_export_other_activate_cb (GtkMenuItem *menuitem,
					 GaleonWindow *window);
void bookmarks_file_ok_button_clicked_cb (GtkButton *button,
					  GaleonWindow *window);
void bookmarks_file_cancel_button_clicked_cb (GtkButton *button,
					      GaleonWindow *window);
void bookmarks_export_file_ok_button_clicked_cb (GtkButton *button, 
						 GaleonWindow *window);
void bookmarks_export_file_cancel_button_clicked_cb (GtkButton *button, 
						     GaleonWindow *window);
void bookmarks_import_netscape_style(GaleonWindow * window, GString * fname);
void bookmark_export_netscape_message (GaleonWindow *window, gchar *fname);
void bookmark_export_netscape_do (gint reply, gchar *fname);


/**
 * Open bookmark in same window if it's a site and the user
 * clicked with button 1
 * Open the bookmark in a new window/tab if it's a site and the user 
 * clicked with button 2
 * Open all the bookmarks in the folder if user clicked with button 2
 * on a folder
 */
gboolean
bookmarks_button_release_event_cb (GtkWidget *widget, GdkEventButton *event,
				   BookMarkItem *bi)
{
	/* FIXME: ugh */
	GtkWidget *WMain = window_lookup_widget (widget, "WMain");
	GaleonWindow *window =  gtk_object_get_data(GTK_OBJECT (WMain), 
						    "GaleonWindow");
	gpointer add_item = gtk_object_get_data (GTK_OBJECT (widget),
						 "add_item");
	gpointer open_all_item = gtk_object_get_data (GTK_OBJECT (widget),
						      "open_all_item");
	gpointer create_toolbar_item = gtk_object_get_data 
						(GTK_OBJECT (widget),
						 "create_toolbar_item");
	GtkWidget *menu;
	static gboolean handled = FALSE;
	gboolean tabbed_mode = gnome_config_get_bool (CONF_APPEARANCE_TABBED);
	gint opts_location = gnome_config_get_int
			       ("/galeon/General/bm_extra_items_location=1");

	return_val_if_not_window (window, FALSE);

	/* Check to see if we're supposed to handle this event */
	/* These checks are here just to make sure tearoff menus get
	   activated correctly */
	if (handled)
	{
		handled = FALSE;
		return FALSE;
	}

	/* If bi is NULL, don't process the event */
	if (!bi)
	{
		handled = TRUE;
		return FALSE;
	}

	/* Get the submenu widget */
	if (GTK_IS_MENU(widget))
		menu = widget;
	else
		menu = widget->parent;

	if (bi->type == SITE)
	{
		gtk_menu_shell_deactivate (GTK_MENU_SHELL(menu));
		if (strstr(bi->url, "%s"))
		{
			bookmarks_smart_bm_dialog (window, bi,
						   (event->button == 2));
		}
		else
		{
			if (event->button == 2)
			{
				if (event->state & GDK_SHIFT_MASK)
					tabbed_mode = !tabbed_mode;
				embed_create_from_url (window->active_embed,
						       bi->url,
						       !tabbed_mode);
			}
			else
			{
				window_load_url (window, bi->url);
			}
		}
	}
	else if (bi->type == AUTOBOOKMARKS || bi->type == CATEGORY)
	{
		/* The "Add bookmark ..." item was clicked */
		if (add_item)
		{
			gtk_menu_shell_deactivate (GTK_MENU_SHELL(menu));
			bookmarks_file_bookmark_cb (widget, bi);
			return TRUE;
		}

		/* The "Open all ..." item was clicked. The current
		   window/tab will be reused when opening the bookmarks */
		if (open_all_item)
		{
			gtk_menu_shell_deactivate (GTK_MENU_SHELL(menu));
			bookmarks_folder_open_all_items (window->active_embed,
							 bi, !tabbed_mode,
							 TRUE);
			return TRUE;
		}

		/* The "Create toolbar ..." item was clicked. */
		if (create_toolbar_item)
		{
			gtk_menu_shell_deactivate (GTK_MENU_SHELL(menu));
			bi->create_toolbar = !bi->create_toolbar;
			bookmarks_save ();
			return TRUE;
		}

		/* A middle click is handled the same way when clicking on
		   the menuitem or directly on a bookmark folder. In tabbed
		   mode, a new window will be opened with new tabs for each
		   bookmark. In new window mode, a new window will be opened
		   for each bookmark.  */
		if ((event->button == 2))
		{
			gtk_menu_shell_deactivate (GTK_MENU_SHELL(menu));
			if (event->state & GDK_SHIFT_MASK)
				tabbed_mode = !tabbed_mode;
			bookmarks_folder_open_all_items (window->active_embed,
							 bi, !tabbed_mode,
							 FALSE);
		}

		/* Display the context menu, if necessary */
		if (event->button == 3 && opts_location == 2)
		{
			GtkWidget *popup = bookmarks_folder_context_menu_new 
						(bi->create_toolbar);
			gint item;

			item = gnome_popup_menu_do_popup_modal (popup, NULL,
								NULL, NULL,
								NULL);
			gtk_widget_destroy (popup);
			gtk_menu_shell_deactivate (GTK_MENU_SHELL(menu));
			switch (item)
			{
			case 0:
				bookmarks_file_bookmark_cb (widget, bi);
				break;
			case 1:
				bookmarks_folder_open_all_items 
						(window->active_embed, bi,
						 !tabbed_mode, TRUE);
				break;
			case 2:
				bi->create_toolbar = !bi->create_toolbar;
				bookmarks_save ();
				break;
			default:
				break;
			}
		}
	}
	return TRUE;
}

static GtkWidget *
bookmarks_folder_context_menu_new (gboolean create_toolbar)
{
	GnomeUIInfo folder_context_menu_uiinfo[] =
	{
		GNOMEUIINFO_ITEM_STOCK (N_("Add bookmark to this folder"),
					NULL, NULL,
					GNOME_STOCK_MENU_BLANK),
		GNOMEUIINFO_ITEM_STOCK (N_("Open all bookmarks in this folder"),
					NULL, NULL,
					GNOME_STOCK_MENU_BLANK),
		GNOMEUIINFO_TOGGLEITEM (N_("Create toolbar from this folder"),
					NULL, NULL,
					GNOME_STOCK_MENU_BLANK),
                GNOMEUIINFO_END
        };
	GtkWidget *context_menu;

	context_menu = gnome_popup_menu_new (folder_context_menu_uiinfo);
	
	gtk_check_menu_item_set_active 
		(GTK_CHECK_MENU_ITEM (folder_context_menu_uiinfo[2].widget),
		 create_toolbar);

	return (context_menu);
}

void
bookmarks_file_bookmark_cb (GtkWidget *widget, BookMarkItem *p)
{
	BookMarkItem *new;	
	GaleonEmbed *embed;
	gchar *url, *name;	
	GtkWidget *WMain;
	GaleonWindow *window;

	WMain = window_lookup_widget (widget , "WMain");
	window = gtk_object_get_data (GTK_OBJECT (WMain), "GaleonWindow");
	return_if_not_window (window);

	embed = window->active_embed;
	return_if_not_embed (embed);

	url = gtk_moz_embed_get_location (embed->mozEmbed);
	name = gtk_moz_embed_get_title (embed->mozEmbed);

	if (name == NULL || strlen(name) == 0)
	{
		if (name) g_free (name);
		name = g_strdup (_("Untitled"));
	}
	new = bookmarks_new_bookmark (SITE, name, url, NULL, NULL, NULL);
	if (url) g_free (url);
	if (name) g_free (name);
	p->list = g_list_append (p->list, new);
	new->parent = p;
	if (bookmarks_editor) bookmarks_editor->dirty = TRUE;

	if (gnome_config_get_int(CONF_GENERAL_BOOKMARK_TITLE) == PAGE_TITLE) {
		if (new->url && 
		    gnome_config_get_bool (CONF_GENERAL_FAVICONS_ENABLED))
			get_siteicon (new->url);

		if (bookmarks_editor)
			bookmarks_editor_place_tree_item (bookmarks_editor,
							  new);
		bookmarks_save ();
	} else {
		gnome_request_dialog 
			(FALSE, _("Please enter the name of the bookmark"),
			 new->name, 100,
			 GTK_SIGNAL_FUNC (bookmarks_string_request_callback),
			 new, GTK_WINDOW (window->WMain));
	}
}

/** 
 * bookmarks_menuitem_drag_data_received_cb::
 */
void
bookmarks_menuitem_drag_data_received_cb (GtkWidget *widget, 
					  GdkDragContext *drag_context,
					  gint x, gint y,
					  GtkSelectionData *selection_data,
					  guint info, guint time,
					  GaleonWindow *window)
{
	BookMarkItem *b = NULL; 
	gchar *mem = selection_data->data;

	switch (info) {
	case DND_TARGET_GALEON_BOOKMARK:
		b = bookmarks_item_from_string (mem);
		break;
	case DND_TARGET_STRING:
	case DND_TARGET_NETSCAPE_URL:
	case DND_TARGET_GALEON_URL:
		b = bookmarks_new_bookmark (SITE, NULL, mem, NULL, NULL, NULL);
		break;
	default:
		g_warning ("Unknown DND type");
		break;
	}
	if (b) {
		/* FIXME */
		GtkMenu *destination_menu;
		if (!bookmarks_editor) bookmarks_editor_init ();
	       	destination_menu = 
			bookmarks_create_copy_menu (bookmarks_editor, b);
		gnome_popup_menu_do_popup_modal
			(GTK_WIDGET (destination_menu), NULL, 
			 NULL, NULL, NULL);
		gtk_widget_destroy (GTK_WIDGET (destination_menu));
		bookmarks_save ();
		bookmarks_free_bookmark (b);
	}
}

/* import given netscape-compatible bookmark filename with pass/fail dialog */
void
bookmarks_import_netscape_style(GaleonWindow * window, GString * fname)
{
	/* NB: used to use gnome_app_message here, but there's not a 
	 * good choice of window to associate the message with, so 
	 * forcing a dialog seems like a better idea -- MattA 30-12-2000 */
	if (netscape_import_bookmarks(fname))
	{
		gnome_ok_dialog (_("Importing bookmarks failed"));
	}
	else
	{
		gnome_ok_dialog (_("Bookmarks successfully imported/merged"));
	}
}

void
bookmarks_import_netscape_activate_cb (GtkMenuItem *menuitem,
				       GaleonWindow *window)
{
	GString *fname = g_string_new (NULL);

	g_string_assign (fname, g_get_home_dir ());
	g_string_append (fname, NETSCAPE_BOOKMARKS);
       	
	bookmarks_import_netscape_style(window, fname);

	g_string_free (fname, TRUE);
}

void
bookmarks_import_mozilla_activate_cb (GtkMenuItem *menuitem,
				      GaleonWindow *window)
{
	GString * fname = g_string_new (NULL);
	char *prefs_dir = get_mozilla_prefs_directory();

	if (!prefs_dir) return;

	g_string_assign (fname, prefs_dir);
	g_string_append (fname, "/bookmarks.html");

	bookmarks_import_netscape_style(window, fname);

	g_string_free (fname, TRUE);
	g_free (prefs_dir);
}

void
bookmarks_import_other_activate_cb (GtkMenuItem *menuitem, GaleonWindow *window)
{
	GtkWidget *fs;
	GladeXML *gxml;

	gxml = glade_widget_new ("bookmarkSelect", window);
	fs = glade_xml_get_widget(gxml, "bookmarkSelect");
	gtk_widget_show(fs);
}


void
bookmarks_file_ok_button_clicked_cb(GtkButton *button, GaleonWindow *window)
{
	gchar * bookmarkFile;
	GString * temp;

	GtkWidget * fs = glade_lookup_widget(GTK_WIDGET(button), "bookmarkSelect");

	bookmarkFile = gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs));
	temp = g_string_new(bookmarkFile);
	bookmarks_import_netscape_style(window, temp);

	gtk_widget_destroy(fs);
	g_string_free(temp, FALSE);
}

void
bookmarks_file_cancel_button_clicked_cb(GtkButton *button, GaleonWindow *window)
{
	GtkWidget *w = glade_lookup_widget(GTK_WIDGET(button), "bookmarkSelect");
	gtk_widget_destroy (w);
}

void
bookmarks_export_netscape_activate_cb (GtkMenuItem *menuitem,
				       GaleonWindow *window)
{
	GString *fname = g_string_new (NULL);

	g_string_assign (fname, g_get_home_dir ());
	g_string_append (fname, NETSCAPE_BOOKMARKS);
       	
	bookmark_export_netscape_message (window, fname->str);

	g_string_free (fname, FALSE);
}

void
bookmarks_export_mozilla_activate_cb (GtkMenuItem *menuitem,
				      GaleonWindow *window)
{
	char *fname;
	char *prefs_dir = get_mozilla_prefs_directory();

	fname = g_strconcat (prefs_dir,"/bookmarks.html", NULL);
	g_print (fname);
	bookmark_export_netscape_message (window, fname);

	g_free (prefs_dir);
}

void
bookmarks_export_other_activate_cb (GtkMenuItem *menuitem, GaleonWindow *window)
{
	GtkWidget *fs;
	GladeXML *gxml;

	gxml = glade_widget_new ("bookmarks_export_file", window);
	fs = glade_xml_get_widget(gxml, "bookmarks_export_file");
	gtk_widget_show(fs);
}

void
bookmarks_export_file_ok_button_clicked_cb (GtkButton *button,
					    GaleonWindow *window)
{
	gchar *bookmarkFile;
	GtkWidget *fs = glade_lookup_widget(GTK_WIDGET(button), "bookmarks_export_file");

	bookmarkFile = gtk_file_selection_get_filename (GTK_FILE_SELECTION(fs));
	if (bookmarkFile) {
		bookmark_export_netscape_message (window, g_strdup (bookmarkFile));
	}
	gtk_widget_destroy(fs);
}

void
bookmarks_export_file_cancel_button_clicked_cb (GtkButton *button,
						GaleonWindow *window)
{
	GtkWidget *w = glade_lookup_widget
		(GTK_WIDGET(button), "bookmarks_export_file");
	gtk_widget_destroy (w);
}

void
bookmark_export_netscape_message (GaleonWindow *window, gchar *fname)
{
	gchar *question = g_strdup_printf (_("File %s will be overwritten.\n"
					     "If you have any bookmark in that file, "
					     "it will be lost.\n\n"
					     "Do you want to continue?"), fname);
	gnome_question_dialog_modal (question,
				     (GnomeReplyCallback) bookmark_export_netscape_do,
				     fname);
	g_free (question);
}

void 
bookmark_export_netscape_do (gint reply, gchar *fname)
{
	if (reply != GNOME_YES) return;
	if (netscape_export_bookmarks (fname, bookmarks_root))
		gnome_ok_dialog (_("Exporting bookmarks failed"));
	else 
		gnome_ok_dialog (_("Bookmarks successfully exported"));
	g_free (fname);
}
