typedef struct
{
	gulong key;
	gint modifier;
} WrapperKeyEventInfo;

typedef struct
{
	gchar *img; 
	gchar *link; 
	gboolean framed_page;
	gint context;
	gint button;
	gint modifier;
} WrapperMouseEventInfo;

typedef struct
{
	const gchar *search_string;
	gboolean backwards;
	gboolean wrap;
	gboolean entire_word;
	gboolean match_case;
} WrapperSearchProperties;


/* Preferences */
extern gboolean mozilla_preference_set (const char *preference_name,
					const char *new_value);
extern gboolean mozilla_preference_set_boolean (const char *preference_name,
						gboolean  new_boolean_value);
extern gboolean mozilla_preference_set_int (const char *preference_name,
					    int new_int_value);

/* Wrapper initialization */
extern gpointer mozilla_wrapper_init (GaleonEmbed *embed,
				      gboolean attach_listener);
extern gboolean mozilla_wrapper_destroy (GaleonEmbed *embed);

/* Base functionalities */
extern gboolean mozilla_find (GaleonEmbed *embed,
			      WrapperSearchProperties *properties);
extern gboolean mozilla_save_image (GaleonEmbed *embed, const char *image, 
				    const char *file_name);
extern gboolean mozilla_save (GaleonEmbed *embed, const char *file_name);
extern gboolean mozilla_save_main_document (GaleonEmbed *embed,
					    const char *file_name);
extern gboolean mozilla_reload (GaleonEmbed *embed);
extern gboolean mozilla_print (GaleonEmbed *embed);
extern gboolean mozilla_set_zoom (GaleonEmbed *embed, float f);
extern gboolean mozilla_get_zoom (GaleonEmbed *embed, float *f);
extern gboolean mozilla_session_history (GaleonEmbed *embed, char **titles[],
					 int *count, int *index);
extern gboolean mozilla_session_history_go (GaleonEmbed *embed, int index);
extern gchar* mozilla_get_document_url (GaleonEmbed *embed);
extern gboolean mozilla_view_source_mode (GaleonEmbed *embed,
					  gboolean view_source);
extern void mozilla_clear_cache (unsigned long cache);

/* Utils */
extern gboolean mozilla_get_key_event_info (GaleonEmbed *embed,
					    gpointer event,
					    WrapperKeyEventInfo *info);
extern gboolean mozilla_get_mouse_event_info (GaleonEmbed *embed,
					      gpointer event,
					      WrapperMouseEventInfo *info);

/* Charsets */
extern void mozilla_get_charsets (GHashTable **charsets,
				  GList **sorted_charset_titles);
extern void mozilla_force_character_set (GaleonEmbed *embed,
					 char *force_character_set);

/* Cookies manager */
extern GList *mozilla_list_cookies (void);
extern GList *mozilla_list_permissions (void);
extern GList *mozilla_list_imagesites (void);
extern void mozilla_block_url (const char *imgURL);
extern void mozilla_set_cookie_permission (GaleonEmbed *embed, gboolean permit);
extern void mozilla_set_image_permission (GaleonEmbed *embed, gboolean permit);
extern gboolean mozilla_set_cookies (GList *gone_c, GList *gone_p,
				     GList *gone_i, gboolean block);

/* Others */
extern gboolean mozilla_copy_session_history (GaleonEmbed *embed,
					      GaleonEmbed *dest);

